MapGen
	var
		zlevel
		size
		numcities
		numlinks
		numhills
		waterlevel
		hill_max_size

		list
			city/gen_cities

	proc
		Generate()
			set background = 1
			world << "Generating world..."
			world << "Initialising..."
			sleep(1)
			for(var/turf/t in block(locate(1,1, zlevel), locate(size, size, zlevel))) t.height = 127
			world << "Done"
			world << "Placing hills and valleys..."
			sleep(1)

			for(var/i=0, i<numhills, i++)
				var/hillx = rand(1, size)
				var/hilly = rand(1, size)
				var/hillsize = rand(1, hill_max_size)
				if(prob(50)) hillsize*=-1
				var/hillsign = 0
				if(hillsize!=0) hillsign = hillsize/abs(hillsize)

				for(var/j = 0, j<abs(hillsize), j++) for(var/turf/t in range(locate(hillx, hilly, zlevel), j)) t.height+=hillsign

			world << "Done"
			world << "Setting basic terrain, finding land..."
			sleep(1)

			var/list/turf/land=list()

			for(var/turf/t in block(locate(1,1, zlevel), locate(size, size, zlevel)))
				t.setFromHeight(waterlevel)
				if(!t.density) land+=t

			world << "Done"
			world << "Placing cities..."
			sleep(1)

			var/list/city/cities=list()

			for(var/i=0, i<numcities, i++)
				var/turf/t = pick(land)
				var/city/c = new /city()
				c.location = t
				cities+=c
				land-=t

			world << "Done"
			world << "Ensuring connectivity... (this may take some time)"
			sleep(1)

			var/list/city/regions=list()
			for(var/city/k in cities)
				var/region/r = new /region()
				r.connected=list(k)
				regions += r

			var/links=0

			while(regions.len>1)
				var/region/r1 = pick(regions)
				var/region/r2 = pick(regions)
				if(r1==r2) continue

				if(!makeLink(pick(r1.connected), pick(r2.connected))) continue
				regions-=r2
				r1.connected+=r2.connected
				links++

			world << "Done"
			world << "Placing additional links..."
			sleep(1)

			while(links<numlinks)
				var/city/c1 = pick(cities)
				var/city/c2 = pick(cities)
				if(c1==c2 || c2 in c1.links) continue
				if(!makeLink(c1, c2)) continue
				links++

			world << "Done"
			world << "Autojoining roads..."
			sleep(1)

			for(var/city/c in cities)
				c.location.icon_state="city"
				c.location.c = c

			gen_cities = cities

			autojoin_road()

			world << "Done"
			world << "Map generation complete!"

		makeLink(city/c1, city/c2)
			set background = 1
			c1.addLink(c2)
			c2.addLink(c1)
			var/turf/start=c1.location
			var/turf/end=c2.location

			var/list/done=list(start)
			var/list/cost=list("\ref[start]"=0)
			var/list/heurist=list("\ref[start]"=get_dist(start,end))
			var/list/previous=list("\ref[start]"=start)
			var/list/next=list()
			for(var/turf/t in get_cardinal(start))
				if(!t.density || t.isWater()) next+=t
				var/co = 1+t.isWater()-t.isRoad()/2
				cost["\ref[t]"] = co
				previous["\ref[t]"] = start
				heurist["\ref[t]"] = get_dist(t, end)

			while(1)
				var/turf/min
				var/mincost = 999999999999
				var/h=0

				for(var/turf/t in next)
					if(cost["\ref[t]"] + heurist["\ref[t]"]<mincost+h)
						min=t
						mincost = cost["\ref[t]"]
						h=heurist["\ref[t]"]

				if(!min) return 0

				done+=min
				next-=min

				if(min==end) break

				for(var/turf/t in get_cardinal(min))
					if(!t.density || t.isWater())
						if(!(t in done+next))
							next+=t
							var/co = mincost+1+t.isWater()-t.isRoad()/2
							cost["\ref[t]"] = co
							heurist["\ref[t]"] = get_dist(t, end)
							previous["\ref[t]"] = min
						else
							if(t in next)
								var/newcost = mincost+1+t.isWater()-t.isRoad()/2
								if(newcost<cost["\ref[t]"])
									cost["\ref[t]"] = newcost
									previous["\ref[t]"] = min

			var/turf/t = end
			while(1)
				t.makeRoad()
				t=previous["\ref[t]"]
				if(t==start) break
			return 1

		heuristic(turf/t1, turf/t2)
			return abs(t1.x-t2.x)+abs(t1.y-t2.y)

		get_cardinal(turf/t)
			return list(get_step(t, NORTH), get_step(t, EAST), get_step(t, SOUTH), get_step(t, WEST))

		autojoin_road()
			for(var/turf/t in block(locate(1,1,zlevel), locate(size,size,zlevel)))
				if(t.isRoad()) t.autojoin()

region
	var/list/city/connected

turf
	icon='terrain.dmi'
	var
		height
		water=0
		road=0
		city/c=null
		joinflag = 0

	proc
		autojoin()
			if(c) return
			match(NORTH, 1)
			match(WEST, 8)
			match(EAST, 2)
			match(SOUTH, 4)
			overlays+=icon('road.dmi', "[joinflag]")

		match(dir, flag)
			var/turf/t = get_step(src, dir)
			if(t && (t.road || t.c))
				joinflag|=flag
				return flag
			else
				joinflag &= ~flag
				return 0

		setFromHeight(waterlevel)
			icon_state="grass"
			water = 0
			road = 0
			joinflag = 0
			c=null
			if(height<waterlevel)
				density=1
				water = 1
				icon_state="shallows"
				if(height<waterlevel/2) icon_state="ocean"
				return
			if(height>waterlevel+32) icon_state="hill"
			if(height>waterlevel+64)
				density=1
				icon_state="mountain"

		makeRoad()
			density = 0
			road = 1

		isWater()
			return water

		isRoad()
			return road

		isCity()
			return c!=null

		getCity()
			return c

	Click()
		if(isCity())
			usr << "<big>This is <b>[c.name]</b>, population <b>[c.population]</b></big>"
			usr << "Media:"
			for(var/media/m in c.companies)	usr << "\t<b>[m.name]</b>, run by <b>[m.execname]</b>"