var
	HELPTEXT = {"
	<html>
	<head><title>Help</title></head>
	<body>
	<big>Introduction</big><br/>
	Media Tycoon is a simulation game. You control an up-and-coming newspaper company, based in a city in the randomly-generated map.
	Your goal is to be the last newspaper company standing - to destroy all opposition, and be the one newspaper company left.
	Media Tycoon was written for the 2008 Game in a Day challenge, and thus is perhaps lacking in normal depth. It uses the "communication" theme.
	What communicates more than the media?<br/><br/>
	<big>Gameplay</big><br/>
	The game interface has four major areas - the map in the top-left corner, the buttons in the middle-left, the output section on the right, and your current status on the bottom left. When you first start, most of these will be empty.
	Press the 'start' button to start the game - it's the furthest to the right, the button that looks like a green circle with "Go" written inside it.
	The game will start generating the map - this can take some time, so give it a few minutes.<br/><br/>
	After the game has started, the other controls will start to do things. Hit the 'pause' button (Two green vertical bars) to stop the game for now.
	The display on the bottom-left of the screen contains all the information about your company - its name, your name, its funds, etc. etc.. The map window now has a display of the area you're in. Green is land, orangey-brown is hills, grey is mountain, and blue and dark blue are water. Cities should be present on the map, linked by roads. Moving the display around using the arrow keys will show you more. Clicking on a city will output information about the population and the companies present.
	You can control the speed of simulation using the four green buttons. The pause button you already know. To its right is the 'slow' speed, then 'normal', and finally 'fast'. To unpause, hit the pause button again. <br/><br/>
	Each simulation step, your company prints some number of newspapers and tries to sell them in a number of towns at a certain price. You can control how many papers you print and what they are sold for. Printing costs money, so be sure to set a price that will earn back the cost of printing. Also try to avoid any wastage - printing papers that can't be sold.
	To control the price you charge for your papers, click on the $-sign button, and then enter a price. If you charge a low price, demand for your paper will surge, while higher prices reap more profit, but may alienate consumers. Your pricing relative to your competition is also important.
	To change the number of papers you print each step, click the # button, and then enter a number.<br/><br/>
	The two remaining buttons are a little more advanced. The briefcase icon allows you to buy out the operations of a newspaper company in a certain city. If you click it, you'll be presented with a list of companies present in all the cities you are in. Clicking on one of them will allow you to buy out their operations - removing their presence from the city. This can be extremely expensive, depending on how well the paper is liked, among other things.
	The other button - looking like a building - allows you to expand your operations to more cities. If you click it, you'll be presented with a list of cities that can be reached directly from where you are. Select one of them, and you'll be presented with the price tag for setting up there, which depends on the competition present. <br/><br/>
	These are the tools you must use to succeed. Sell your papers for an appropriate price, expand, and buy out your competitors. Become the sole communicator in this land.<br/><br/>
	\t--Jp
	</body
	</html>
	"}

world
	maxx=100
	maxy=100
	view = "28x21"

mob
	verb
		test()


mob
	density = 0

client
	var
		media/company
		game/sim
		obj/pause_disp
		obj/speedind

	New()
		.=..()
		pause_disp = new /obj/pauseobj()
		speedind = new /obj/normalspeed()

	verb
		help()
			src << browse(HELPTEXT, "window=help")
			return
		fast()
			if(sim)
				sim.simdelay = 10
				screen-=speedind
				speedind = new /obj/fastspeed()
				if(!sim.pause) screen+=speedind

		normal()
			if(sim)
				sim.simdelay = 100
				screen-=speedind
				speedind = new /obj/normalspeed()
				if(!sim.pause)screen+=speedind

		slow()
			if(sim)
				sim.simdelay = 1000
				screen-=speedind
				speedind = new /obj/slowspeed()
				if(!sim.pause)screen+=speedind

		pause()
			if(sim)
				sim.pause=!sim.pause
				if(sim.pause)
					screen+=pause_disp
					screen-=speedind
				else
					screen-=pause_disp
					screen+=speedind

		start()
			if(!sim)
				var/game/g = new /game(1, src)
				if(!g) return

				g.start_simloop()

		papers()
			if(!sim) return
			var/n = input("How many papers would you like to print each run?", "Number of papers printed") as num|null
			if(n==null || !company) return
			n=round(n)
			if(n<0) n=0
			company.num_papers=n

			sim.print_data()

		price()
			if(!sim) return
			var/p = input("What would you like to charge for each paper?", "Newspaper price") as num|null
			if(p==null || !company) return
			if(p<0) p=0
			company.price = p

			sim.print_data()

		setup()
			if(!sim) return
			var/list/poss = list("Cancel")
			for(var/city/c in sim.cities)
				if(!(company in c.companies)) continue
				for(var/city/c2 in c.links)
					if(c2 in poss) continue
					if(!(company in c2.companies))
						poss+=c2

			if(poss.len==1) world << "There aren't any cities you can expand into"
			var/city/c = input("Which city would you like to expand into?", "Expanding") in poss
			if(!c || c=="Cancel" || !company || !sim) return

			var/cost = 1000
			for(var/media/m in c.companies)	cost+=100*c.demand["\ref[m]"]
			cost = round(cost, 0.1)

			var/rep = alert("This will cost $[cost]. Are you sure?", "Expanding into [c]","Yes", "No")
			if(rep=="No" || !company || !sim) return

			company.money-=cost

			c.companies+=company

			sim.print_data()

		buyout()
			if(!sim) return
			var/list/cits=list()
			for(var/city/c in sim.cities) if((company in c.companies) && c.companies.len>1) cits+=c
			var/list/city_by_option = list()
			var/list/media_by_option = list()

			var/list/poss = list("Cancel")

			for(var/city/c in cits)
				for(var/media/m in c.companies)
					if(m==company) continue

					poss += "[m], in [c]"

					city_by_option["[m], in [c]"] = c
					media_by_option["[m], in [c]"] = m

			if(poss.len==1)	world << "There aren't any companies you can buy out"

			var/rep = input("Which newspaper office would you like to buy out?", "Buy out a company") in poss
			if(!rep || rep=="Cancel"|| !company || !sim) return

			var/city/c = city_by_option[rep]
			var/media/m = media_by_option[rep]

			var/cost = m.money*c.demand["\ref[m]"]/10

			if(cost<1000) cost=1000

			rep = alert("This will cost $[cost]. Are you sure?", "Buying out [rep]", "Yes", "No")
			if(rep=="No"|| !company || !sim) return

			company.money-=cost
			c.companies-=m

			for(c in sim.cities)
				if(m in c.companies) return

			sim.outOfBusiness(m)
			sim.print_data()

game
	var
		list/media/bought_out
		list/media/companies = list()
		list/city/cities = list()
		client/player
		day=1
		pause = 0
		simdelay = 100

	New(zlevel, client/p)
		var/MapGen/g = new /MapGen()

		g.size=100
		g.zlevel=zlevel
		g.waterlevel=128
		g.numcities = 20
		g.numlinks = 30
		g.numhills = 50
		g.hill_max_size=32
		var/e = input(p, "What is your name?") as text|null
		if(!e) return 0
		var/n = input(p, "What would you like to call your newspaper?") as text|null
		if(!n) return 0
		g.Generate()

		cities = g.gen_cities
		for(var/city/c in cities) companies += c.companies

		var/city/c = pick(cities)
		var/media/m = new /media(c, p, n, e)
		c.companies+=m
		c.demand["\ref[m]"] = 50
		p.company=m
		p.sim = src
		companies+=m
		player = p
		p.mob.loc = c.location

	proc
		outOfBusiness(media/m)
			if(!bought_out) bought_out = list()
			bought_out+=m

		print_data()
			var/ret = "<big><b>[player.company]</b> News Ltd. Annual Report, Period <b>[day]</b>:<br />Presiding CEO, [player.company.execname]</big><br />"
			ret+="Total Funds: <b>$[player.company.money]</b><br />"
			ret+="Papers Sold last Annum: <b>[player.company.numsold]</b><br /><br />"
			ret+="<big>Plans:</big><br />"
			ret+="Pricing next annum: <b>$[player.company.price]</b> per newspaper<br />"
			ret+="Printing <b>[player.company.num_papers]</b> newspapers for sale. This will cost <b>$[player.company.num_papers*player.company.cost].</b><br /><br />"
			ret+="<big>Business expansion report:</big><br />"
			ret+="We are present in the following cities:<br />"

			for(var/city/c in cities)
				if(player.company in c.companies)
					ret+="\t<b>[c]</b>, population <b>[c.population]</b>. There are <b>[c.companies.len-1?c.companies.len-1:"No"]</b> competitiors. We sold <b>[c.numsold["\ref[player.company]"]?c.numsold["\ref[player.company]"]:"no"]</b> papers here last annum<br/><br/>"

			player << browse(ret)

		start_simloop()
			if(!player)
				del src
				return
			print_data()
			simloop:
				while(1)
					while(pause) sleep(10)
					sleep(simdelay)
					sim_step()

					for(var/media/m in companies)
						if(m.money<=-10000 || (m in bought_out))
							world << "<big>Year [day]:</big> <b>[m.name]</b> has gone out of business!"
							for(var/city/c in cities) if(m in c.companies) c.companies-=m
							companies-=m
							if(m==player.company)
								world << "<big>You've run out of luck. Your business has gone down the plughole. Game over.</big>"
								print_data()
								del m
								break simloop
							del m

					bought_out = null

					if(companies.len<=1 && (player.company in companies))
						world << "<big>You've done it. It's taken hard work, and a lot of time, but you've become the only newspaper company in the land</big>"
						del player.company
						break simloop

					print_data()
					day++
			del src

		sim_step()
			for(var/media/m in companies)
				m.temp_num_papers = m.num_papers
				m.temp_money = m.money
				m.money-=m.cost*m.num_papers

			for(var/city/c in cities) c.sim_step()

			for(var/media/m in companies)
				m.numsold = m.num_papers-m.temp_num_papers
				m.moneychange = m.money-m.temp_money
				if(m!=player.company) Ai(m)

		Ai(media/m)
			if(m.moneychange<0) //Losing money!
				if(!(m.num_papers>m.numsold*1.2)) //If we didn't make too many papers, our problem is pricing
					if(m.price>1.8) //We're charging too much and making ourselves unpopular
						m.price-=rand(1,8)/10 //Lower it.
					else
						if(m.price<1.2) //We're charging too little and running out of money
							m.price+=rand(1,8)/10 //Raise it

			m.num_papers = max(1,(m.numsold*1.1))

			if(m.money>1000) //If you have some money...
				//Consider setting up somewhere else
				if(prob(50))
					var/list/poss = list()
					for(var/city/c in cities)
						if(!(m in c.companies)) continue
						for(var/city/c2 in c.links)
							if(c2 in poss) continue
							if(!(m in c2.companies))
								poss+=c2

					if(poss.len) //If there's somewhere to expand to...
						var/city/c = pick(poss)

						var/cost = 1000
						for(var/media/m2 in c.companies)	cost+=100*c.demand["\ref[m2]"]
						cost = round(cost, 0.1)

						if(cost<m.money+5000) //Check we can afford it...
							m.money-=cost	//And do it!
							c.companies+=m
							world << "<b>[m]</b> expanded to include <b>[c]</b> in its distribution"
				else
					if(prob(50)) //Consider buying someone out
						var/list/cits=list()
						for(var/city/c in cities) if((m in c.companies) && c.companies.len>1) cits+=c
						var/list/city_by_option = list()
						var/list/media_by_option = list()

						var/list/poss = list()

						for(var/city/c in cits)
							for(var/media/m2 in c.companies)
								if(m2==m) continue

								poss += "[m2], in [c]"

								city_by_option["[m2], in [c]"] = c
								media_by_option["[m2], in [c]"] = m2

						if(poss.len)
							var/rep = pick(poss)

							var/city/c = city_by_option[rep]
							var/media/m2 = media_by_option[rep]

							var/cost = m2.money*c.demand["\ref[m2]"]/10

							if(cost<1000) cost=1000

							if(cost<m.money+5000)
								m.money-=cost
								c.companies-=m2

								world << "<b>[m]</b> bought out <b>[m2]'s</b> operations in <b>[c]</b>"
								for(c in cities) if(m2 in c.companies) return
								outOfBusiness(m2)
				print_data()

obj
	pauseobj
		icon='pause.dmi'
		screen_loc="NORTH, WEST"

	fastspeed
		screen_loc="NORTH, WEST"
		icon='fast.dmi'

	normalspeed
		screen_loc="NORTH, WEST"
		icon='normal.dmi'

	slowspeed
		screen_loc="NORTH, WEST"
		icon='slow.dmi'